from .core.element import HTMLElement
from .tags.form import *
from .tags.html import *
from .tags.layout import *
from .tags.lists import *
from .tags.media import *
from .tags.table import *
from .tags.text import *